@@ -4,19 +4,19 @@ BSD 3-Clause License
 Copyright (c) 2020, Daniel Nagel
 All rights reserved.
 
-Author: Daniel Nagel
-
 """
 # ~~~ IMPORT ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-import os.path
+from os import path
+import warnings
 
 import matplotlib as mpl  # mpl = dm.tryImport('matplotlib')
-import matplotlib.legend as mlegend
-import matplotlib.pyplot as plt
-import mpl_toolkits.axes_grid1
+import numpy as np
+from matplotlib import legend as mlegend
+from matplotlib import pyplot as plt
+from mpl_toolkits import axes_grid1 as mpl_axes_grid1
 
 from prettypyplot import _tools
-from prettypyplot.style import __MODE, __STYLE, __get_scale
+from prettypyplot.style import __MODE, __STYLE
 
 
 # ~~~ FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -41,9 +41,7 @@ def imshow(*args, ax=None, **kwargs):
         kwargs['zorder'] = 1
 
     # plot
-    im = ax.imshow(*args, **kwargs)
-
-    return im
+    return ax.imshow(*args, **kwargs)
 
 
 def plot(*args, ax=None, **kwargs):
@@ -116,27 +114,41 @@ def savefig(fname, use_canvas_size=True, **kwargs):
         See [pyplot.savefig()](MPL_DOC.pyplot.savefig.html)
 
     """
-    fig, ax = plt.gcf(), plt.gcf().get_axes()[0]
+    fig = plt.gcf()
+    ax = plt.gcf().get_axes()[0]
     figsize = fig.get_size_inches()
 
     set_figsize = figsize
 
     if __STYLE == 'minimal':
         # reduce number of ticks by factor 1.5 if more than 4
+        tick_reduc = 1.5
         for axes in plt.gcf().get_axes():
             if len(axes.get_xticks()) > 4:
-                axes.locator_params(tight=False, axis='x',
-                                    nbins=len(axes.get_xticks()) / 1.5)
+                axes.locator_params(
+                    tight=False,
+                    axis='x',
+                    nbins=len(axes.get_xticks()) / tick_reduc,
+                )
             if len(axes.get_yticks()) > 4:
-                axes.locator_params(tight=False, axis='y',
-                                    nbins=len(axes.get_yticks()) / 1.5)
+                axes.locator_params(
+                    tight=False,
+                    axis='y',
+                    nbins=len(axes.get_yticks()) / tick_reduc,
+                )
 
     if __MODE == 'poster':
-        fig.set_size_inches((3 * figsize[0], 3 * figsize[1]))
+        fig.set_size_inches(
+            (3 * figsize[0], 3 * figsize[1]),
+        )
     elif __MODE == 'beamer':
-        fig.set_size_inches((3 * figsize[0], 3 * figsize[1]))
+        fig.set_size_inches(
+            (3 * figsize[0], 3 * figsize[1]),
+        )
 
-    fig.tight_layout(pad=0.20, h_pad=0.00, w_pad=0.00)
+    with warnings.catch_warnings():
+        warnings.filterwarnings('ignore')
+        fig.tight_layout()
 
     # convert figsize to canvas size
     if use_canvas_size:
@@ -146,10 +158,8 @@ def savefig(fname, use_canvas_size=True, **kwargs):
 
     # save as pdf if not specified
     if 'format' not in kwargs:
-        fmt = os.path.splitext(fname)[1][1:]
-        if format == '':
-            fmt = 'pdf'
-            fname = '{0}.{1}'.format(fname, fmt)
+        if path.splitext(fname)[1][1:] == '':
+            fname = '{0}.pdf'.format(fname)
 
     # save fig
     plt.savefig(fname, **kwargs)
@@ -239,7 +249,7 @@ def legend(*args, outside=False, ax=None, axs=None, **kwargs):
     if __STYLE == 'minimal':
         leg.get_frame().set_linewidth(0.)
     elif __STYLE == 'default':
-        leg.get_frame().set_linewidth(__get_scale(__MODE)['small_scale'] * 0.8)
+        leg.get_frame().set_linewidth(plt.rcParams['axes.linewidth'])
 
     # shift title to the left if on top or bottom
     # taken from: https://stackoverflow.com/a/53329898
@@ -339,7 +349,7 @@ def colorbar(im, width='7%', pad='0%', position='right', label=None, **kwargs):
         ax = plt.gca()
 
     # generate divider
-    divider = mpl_toolkits.axes_grid1.make_axes_locatable(ax)
+    divider = mpl_axes_grid1.make_axes_locatable(ax)
     cax = divider.append_axes(position, width, pad=pad)
 
     cbar = plt.colorbar(im, cax=cax, orientation=orientation)
@@ -402,3 +412,134 @@ def __minmax(lim, rcparam):
     margin = plt.rcParams[rcparam]
     minmax = [lim[0] + (margin + i) / (1 + 2 * margin) * width for i in [0, 1]]
     return minmax
+
+
+def hide_empty_axes(axs=None):
+    """Hide empty axes.
+
+    Loop over all axes and hide empty ones.
+
+    Parameters
+    ----------
+    axs : mpl.axes.Axes or list of
+        Specify axes to check for empty state. Default use all of current
+        figure.
+
+    """
+    # check for single axes
+    if axs is not None:
+        if isinstance(axs, mpl.axes.Axes):
+            axs = [axs]
+        elif all((
+            isinstance(arg, mpl.axes.Axes) for arg in np.ravel(axs)
+        )):
+            axs = np.ravel(axs)
+        else:
+            raise TypeError('axs needs to be of type matplotlib.axes.Axes.')
+    else:
+        axs = plt.gcf().get_axes()
+
+    # loop over all axes and hide empty ones
+    for ax in axs:
+        if _is_empty_axes(ax):
+            ax.axis('off')
+
+
+def label_outer(axs=None):
+    """Only show outer labels and tick labels.
+
+    This checks for outest visible axes only. Works only with single Gridspec.
+
+    Parameters
+    ----------
+    axs : list of mpl.axes.AxesSubplot
+    """
+    # check for single axes
+    if axs is not None:
+        if _is_subplot_axes(axs):
+            axs = [axs]
+        elif all((_is_subplot_axes(arg) for arg in np.ravel(axs))):
+            axs = np.ravel(axs)
+        else:
+            raise TypeError(
+                'axs needs to be of type matplotlib.axes.AxesSuplot.',
+            )
+    else:
+        axs = [ax for ax in plt.gcf().get_axes() if _is_subplot_axes(ax)]
+
+    for ax in axs:
+        ss = ax.get_subplotspec()
+        if hasattr(ss, 'is_last_row'):  # noqa: WPS421
+            # for mpl >= 3.4
+            lastrow = ss.is_last_row()
+            firstcol = ss.is_first_col()
+        else:
+            lastrow = ax.is_last_row()
+            firstcol = ax.is_first_col()
+
+        # check if axes below, left is hidden
+        left_empty, bottom_empty = _is_outer_empty(axs, ax)
+        _label_outer(ax, lastrow or bottom_empty, firstcol or left_empty)
+
+
+def _is_subplot_axes(ax):
+    """Check is is subplot axes."""
+    return (
+        isinstance(ax, mpl.axes.Axes) and
+        hasattr(ax, 'get_subplotspec')  # noqa: WPS421
+    )
+
+
+def _is_outer_empty(axs, ax):
+    """Check if lefter/lower axes is empty."""
+    left_empty, bottom_empty = False, False
+    for axes in axs:
+        if _is_left_neighbor(axes, ax):
+            left_empty = left_empty or not axes.axison
+
+        if _is_bottom_neighbor(axes, ax):
+            bottom_empty = bottom_empty or not axes.axison
+    return left_empty, bottom_empty
+
+
+def _is_left_neighbor(ax1, ax2):
+    """Check if ax1 is left ax2."""
+    return _has_neighbor_distance(ax1, ax2, col_offset=1)
+
+
+def _is_bottom_neighbor(ax1, ax2):
+    """Check if ax1 is below ax2."""
+    return _has_neighbor_distance(ax1, ax2, row_offset=-1)
+
+
+def _has_neighbor_distance(ax1, ax2, row_offset=0, col_offset=0):
+    """Check if two SubpotAxes have the given offset distance."""
+    ss1, ss2 = ax1.get_subplotspec(), ax2.get_subplotspec()
+    return (
+        any((
+            row + row_offset in list(ss2.rowspan) for row in list(ss1.rowspan)
+        )) and any((
+            col + col_offset in list(ss2.colspan) for col in list(ss1.colspan)
+        ))
+    )
+
+
+def _label_outer(ax, lastrow, firstcol):
+    """See mpl.axes.Axes.label_outer()."""
+    if not lastrow:
+        for xlabel in ax.get_xticklabels(which='both'):
+            xlabel.set_visible(False)
+        ax.xaxis.get_offset_text().set_visible(False)
+        ax.set_xlabel('')
+    if not firstcol:
+        for ylabel in ax.get_yticklabels(which='both'):
+            ylabel.set_visible(False)
+        ax.yaxis.get_offset_text().set_visible(False)
+        ax.set_ylabel('')
+
+
+def _is_empty_axes(ax):
+    """Return if axes is empty."""
+    return (
+        not ax.lines and not ax.collections and not ax.patches and not ax.texts
+    )
